/*******************************************************************************
* Copyright (C) 2004-2006 Intel Corp. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Intel Corp. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Intel Corp. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*******************************************************************************/

/**
 * @author Anas Nashif
 * @author Eugene Yarmosh
 */


//
// WsManTest2_data.h
//
// This file is intended to be eventually generated by a codegen tool
//
#ifndef __WS_MAN_TEST_H__
#define __WS_MAN_TEST_H__

#define WS_MAN_TEST_RESOURCE_URI		"wsman:system/2005/06/test2"



// The resource is modeled as a struct
struct __WsManTest2
{
	char* Test;
};
typedef struct __WsManTest2 WsManTest2;


// Service endpoint declaration
int WsManTest2_Enumerate_EP(WsContextH cntx, WsEnumerateInfo* enumInfo);
int WsManTest2_Release_EP(WsContextH cntx, WsEnumerateInfo* enumInfo);
int WsManTest2_Pull_EP(WsContextH cntx, WsEnumerateInfo* enumInfo);
int  WsManTest2_Put_EP(WsContextH cntx, void *in, WsManTest2 **out);
WsManTest2* WsManTest2_Get_EP (WsContextH cntx);




// Two macro below will declare 
// Serialization type info declaration for the resource
// extern struct __XmlSerializerInfo WsManTest2_TypeInfo[];
// End point array declaration for the resource
// extern WsDispatchEndPointInfo WsManTest2_EndPoints[];
SER_DECLARE_TYPE(WsManTest2);
DECLARE_EP_ARRAY(WsManTest2);
DECLARE_SELECTOR_ARRAY(WsManTest2);



void get_endpoints(GModule *self, void **data);
int init (GModule *self, void **data );
void cleanup( GModule *self, void *data );
#endif //__WS_MAN_TEST_H__
